/* 
Copyright (C) 2004 Jon Snell, jsnell@e-normous.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

this file contains most of the code to work with the modem.
it:
inits the modem
does modem i/o


TODO:
separate the gtk code out of this, make it work 
try all init strings or let the user pick

*/

#include <stdio.h>   /* Standard input/output definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */

#include "caller_id.h"
#include "modem.h"



int
open_port(void)
{
  int fd; /* File descriptor for the port */
  struct termios options; /* for setting options */  
  
  fd = open(MODEM_DEVICE, O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd == -1)
    {
      return FALSE;
    }
  else
    {
      fcntl(fd, F_SETFL, FNDELAY);
      //      tcgetattr(fd, &options);
      bzero(&options, sizeof(options));
      cfsetispeed(&options, BAUDRATE);
      cfsetospeed(&options, BAUDRATE);
      options.c_cflag &= ~CSTOPB; /* */
      options.c_cflag &= ~PARENB; 
      options.c_cflag &= ~CSIZE;
      options.c_cflag |= CS8;  /* 8 data bits */
      options.c_cflag &= ~CLOCAL;  /* we want flow control       */ 
      // CLOCAL 
      options.c_cflag |= (CREAD | CRTSCTS);
      options.c_iflag |= (ICANON | ECHO | ECHOE); /* turn on echo */
      options.c_iflag |= IGNPAR | ICRNL; /* ignore parity, map carriage return to newline */
      tcflush(fd, TCIFLUSH);
      tcsetattr(fd, TCSANOW, &options);

    }
 


  return (fd);
}

int modem_write(CIDData *cid, const char *str)
{
  int n;
#ifdef MODEM_DEBUG
  FILE *debug_f;

  debug_f = fopen("/tmp/cid_debug.txt", "a+");
  fprintf(debug_f, "w: %s\n", str);
  fclose(debug_f);
#endif
	      
  n = write(cid->modem, str, strlen(str));
  if (n < 0)
    gtk_label_set_text((GtkLabel *)cid->label, "write() failed!\n");
}

int modem_read(CIDData *cid)
{
  char tmp_buf [2];
  int n;

#ifdef MODEM_DEBUG
  char debug_buf[500];
  FILE *debug_f;
#endif

  tmp_buf[0] = '\0';
  
    n = read(cid->modem, tmp_buf, 1);
    {
      if (n > 0)
	{
	  tmp_buf[1] = 0; // terminate it
	  if (tmp_buf[0] == '\n')
	    {
	      cid->line_complete = 1;
#ifdef MODEM_DEBUG
	      debug_f = fopen("/tmp/cid_debug.txt", "a+");
	      fprintf(debug_f, "r: CR\n");
	      fclose(debug_f);
#endif
	    }
	  else if (tmp_buf[0] == '\r')
	    {
	      // ignore
	    }
	  else
	    {
#ifdef MODEM_DEBUG
	      debug_f = fopen("/tmp/cid_debug.txt", "a+");
	      fprintf(debug_f, "r: %s\n", tmp_buf);
	      fclose(debug_f);
#endif
	      strncat(cid->input_buf, tmp_buf, sizeof(cid->input_buf));
	    }
	  
	}
    }

      
}

char *modem_gets(CIDData *cid)
{
  modem_read(cid);
  
  if (cid->line_complete != 0)
    {
      strcpy(cid->input_str, cid->input_buf);
      cid->input_buf[0] = 0;
      cid->line_complete = 0;
      return (char *) &cid->input_str;
    }
  return NULL;
}

int init_modem(CIDData *cid)
{
  char buf[1024];
char *m_data;
  if (cid->initialize_step == 0)
    {
      cid->modem = open_port();
      if (cid->modem)
	gtk_label_set_text((GtkLabel *)cid->label, "Open");
      else
	{
	  gtk_label_set_text((GtkLabel *)cid->label, "Failed Opening Modem");
	  
	}

      modem_write(cid,INIT_STRING);
      cid->initialize_step++;
    }
  else if (cid->initialize_step == 1)
    {
      // empty...
      cid->initialize_step++;
    }
  else if (cid->initialize_step == 2)
    {
      cid->initialize_step++;
    }
  else if (cid->initialize_step == 3)
    {
      // waiting for OK
      m_data = modem_gets(cid);
      if (m_data)
	if (strcmp(m_data, "OK"))
	  {
	    gtk_label_set_text((GtkLabel *)cid->label, "Init OK");
	    cid->initialize_step++;	    
	    modem_write(cid, CID_INIT_STRING);
	  }
    }
  else if (cid->initialize_step == 4)
    {
      // waiting for OK
      m_data = modem_gets(cid);
      if (m_data)
	if (strcmp(m_data, "OK"))
	  {
	    gtk_label_set_text((GtkLabel *)cid->label, "CID Init OK");
	    cid->initialize_step++;	    
	  }
    }
  else
    {
      gtk_label_set_text((GtkLabel *)cid->label, (const char *)"Waiting");
      cid->initialized = 1;
    }

  
}

int parse_modem_text(char *m_text, CIDRecord *cid_record)
{
  /* format: 
DATE = 0822
TIME = 1812
NMBR = 0009271317
NAME = CELLULAR CALL
  */

  if (strstr(m_text,CID_NAME_PREFIX))
    {
      cid_record->partial = 1;
      strncpy(cid_record->name, (const char *) &m_text[strlen(CID_NAME_PREFIX)],32);
    }
  else if (strstr(m_text,CID_NUMBER_PREFIX))
    {
      cid_record->partial = 1;
      strncpy(cid_record->nmbr, (const char *) &m_text[strlen(CID_NUMBER_PREFIX)], 32);
    }
  else if (strstr(m_text,"RING"))
    {
      cid_record->ring_count++;;
      if (cid_record->partial == 1) // if it was partially filled out, then there is another ring, it's done
	cid_record->complete = 1; 
      else if (cid_record->ring_count > 1)
	{
	  cid_record->complete = 1;
	  cid_record->error = 1; 
	}
      
    }
}
